@@ -108,4 +108,6 @@ class SelectionOp(BaseOperator):
             The schemas of the columns produced by this operator
         """
         selector = col_selector or self.selector
+        if selector.all:
+            selector = ColumnSelector(input_schema.column_names)
         return super().compute_output_schema(input_schema, selector, prev_output_schema)
