@@ -14,26 +14,24 @@
 # limitations under the License.
 #
 
-from dataclasses import dataclass, field
-from enum import Enum
-from typing import Dict, List, Optional, Text, Union
+from dataclasses import InitVar, dataclass, field, replace
+from typing import Dict, List, Optional, Text, Tuple, Union
 
-import numpy as np
 import pandas as pd
 
+import merlin.dtypes as md
+from merlin.dtypes import DType
+from merlin.dtypes.shape import Shape
 from merlin.schema.tags import Tags, TagSet
 
 
-class ColumnQuantity(Enum):
-    """Describes the number of elements in each row of a column"""
-
-    SCALAR = "scalar"
-    FIXED_LIST = "fixed_list"
-    RAGGED_LIST = "ragged_list"
-
-
 @dataclass(frozen=True)
 class Domain:
+    """Describes an integer or float domain.
+
+    Can be partially specified. With any of name, min, max.
+    """
+
     min: Optional[Union[int, float]] = None
     max: Optional[Union[int, float]] = None
     name: Optional[str] = None
@@ -55,90 +53,74 @@ class ColumnSchema:
     """A schema containing metadata of a dataframe column."""
 
     name: Text
-    tags: Optional[TagSet] = field(default_factory=TagSet)
+    tags: Optional[Union[TagSet, List[Union[str, Tags]]]] = field(default_factory=TagSet)
     properties: Optional[Dict] = field(default_factory=dict)
-    dtype: Optional[object] = None
+    dtype: Optional[DType] = None
     is_list: Optional[bool] = None
     is_ragged: Optional[bool] = None
+    dims: InitVar[Union[Tuple, Shape]] = None
 
-    def __post_init__(self):
+    def __post_init__(self, dims):
         """Standardize tags and dtypes on initialization
 
+        This method works around the inability to set attributes on frozen dataclass
+        objects by using object.__setattr__, which bypasses the methods that frozen
+        dataclasses lock down. That approach allows to do some normalization on the
+        object's attribute values in the post init hook that we otherwise wouldn't
+        have a way to implement.
+
         Raises:
             TypeError: If the provided dtype cannot be cast to a numpy dtype
+            ValueError: If the provided shape, value counts, and/or flags are inconsistent
         """
-        tags = TagSet(self.tags)
-        object.__setattr__(self, "tags", tags)
-
-        try:
-            if hasattr(self.dtype, "numpy_dtype"):
-                dtype = np.dtype(self.dtype.numpy_dtype)
-            elif hasattr(self.dtype, "_categories"):
-                dtype = self.dtype._categories.dtype
-            elif isinstance(self.dtype, pd.StringDtype):
-                dtype = np.dtype("O")
-            else:
-                dtype = np.dtype(self.dtype)
-        except TypeError as err:
-            raise TypeError(
-                f"Unsupported dtype {self.dtype}, unable to cast {self.dtype} to a numpy dtype."
-            ) from err
+        # Provide defaults and minor conversions for convenience
+        object.__setattr__(self, "tags", TagSet(self.tags))
 
+        dtype = md.dtype(self.dtype or md.unknown).without_shape
         object.__setattr__(self, "dtype", dtype)
 
-        value_count = Domain(**self.properties.get("value_count", {}))
+        # Validate that everything provided is consistent
+        value_counts = self.properties.get("value_count", {})
+        if self.is_list and self.is_ragged is False:
+            if "max" in value_counts and "min" not in value_counts:
+                value_counts["min"] = value_counts["max"]
+            if "max" not in value_counts and "min" in value_counts:
+                value_counts["max"] = value_counts["min"]
 
-        if value_count.min == 0 or value_count.max == 0:
-            raise ValueError(
-                "`value_count` min and max must be greater than zero. "
-                f"Provided min: {value_count.min} max: {value_count.max}"
-            )
+        self._validate_shape_info(self.shape, value_counts, self.is_list, self.is_ragged)
 
-        if self.is_list is None:
-            if value_count.max and value_count.max > 0:
-                object.__setattr__(self, "is_list", True)
-            else:
-                object.__setattr__(self, "is_list", False)
+        # Pick which source to pull shape info from
+        if dims:
+            new_shape = Shape(dims)
+        elif dtype.shape.dims:
+            new_shape = dtype.shape
+        elif value_counts:
+            new_shape = self._shape_from_counts(Domain(**value_counts))
+        elif self.is_list:
+            new_shape = self._shape_from_flags(self.is_list)
+        else:
+            new_shape = Shape()
 
-        if self.is_ragged is None:
-            if value_count.is_bounded and value_count.max > value_count.min:
-                object.__setattr__(self, "is_ragged", True)
-            elif value_count.is_bounded and value_count.max == value_count.min:
-                object.__setattr__(self, "is_ragged", False)
-            else:
-                object.__setattr__(self, "is_ragged", self.is_list)
+        # Update the shape and propagate out to flags and value counts
+        dtype = dtype.with_shape(new_shape)
+        object.__setattr__(self, "dtype", dtype)
+        object.__setattr__(self, "is_list", dtype.shape.is_list)
+        object.__setattr__(self, "is_ragged", dtype.shape.is_ragged)
 
-        if self.is_ragged and not self.is_list:
-            raise ValueError(
-                "`is_ragged` is set to `True` but `is_list` is not. "
-                "Only list columns can set the `is_ragged` flag."
-            )
+        if new_shape.dims is not None and len(new_shape.dims) > 1:
+            value_counts = {"min": new_shape.dims[1].min, "max": new_shape.dims[1].max}
+            properties = {**self.properties, **{"value_count": value_counts}}
+            object.__setattr__(self, "properties", properties)
 
-        if self.is_ragged and value_count.is_bounded and value_count.min == value_count.max:
-            raise ValueError(
-                "`is_ragged` is set to `True` but `value_count.min` == `value_count.max`. "
-                "If value_count min/max are equal. "
-                "This is a fixed size list and `is_ragged` should be set to False. "
-            )
+    def _shape_from_flags(self, is_list):
+        return Shape(((0, None), (0, None))) if is_list else None
+
+    def _shape_from_counts(self, value_count):
+        return Shape(((0, None), (value_count.min or 0, value_count.max)))
 
     @property
-    def quantity(self):
-        """
-        Describes the number of elements in each row of this column
-
-        Returns
-        -------
-        ColumnQuantity
-            SCALAR when one element per row
-            FIXED_LIST when the same number of elements per row
-            RAGGED_LIST when different numbers of elements per row
-        """
-        if self.is_list and self.is_ragged:
-            return ColumnQuantity.RAGGED_LIST
-        elif self.is_list:
-            return ColumnQuantity.FIXED_LIST
-        else:
-            return ColumnQuantity.SCALAR
+    def shape(self):
+        return self.dtype.shape
 
     def with_name(self, name: str) -> "ColumnSchema":
         """Create a copy of this ColumnSchema object with a different column name
@@ -154,14 +136,7 @@ class ColumnSchema:
             Copied object with new column name
 
         """
-        return ColumnSchema(
-            name,
-            tags=self.tags,
-            properties=self.properties,
-            dtype=self.dtype,
-            is_list=self.is_list,
-            is_ragged=self.is_ragged,
-        )
+        return replace(self, name=name)
 
     def with_tags(self, tags: Union[str, Tags]) -> "ColumnSchema":
         """Create a copy of this ColumnSchema object with different column tags
@@ -177,14 +152,7 @@ class ColumnSchema:
             Copied object with new column tags
 
         """
-        return ColumnSchema(
-            self.name,
-            tags=self.tags.override(tags),
-            properties=self.properties,
-            dtype=self.dtype,
-            is_list=self.is_list,
-            is_ragged=self.is_ragged,
-        )
+        return replace(self, tags=self.tags.override(tags))  # type: ignore
 
     def with_properties(self, properties: dict) -> "ColumnSchema":
         """Create a copy of this ColumnSchema object with different column properties
@@ -206,24 +174,26 @@ class ColumnSchema:
 
         """
         if not isinstance(properties, dict):
-            raise TypeError("properties must be in dict format, key: value")
+            raise TypeError("ColumnSchema properties must be a dictionary")
 
         # Using new dictionary to avoid passing old ref to new schema
         new_properties = {**self.properties, **properties}
 
-        is_ragged = self.is_ragged
-        value_count = Domain(**new_properties.get("value_count", {}))
-        if value_count.is_bounded and value_count.max == value_count.min:
-            is_ragged = False
+        value_counts = properties.get("value_count", {})
 
-        return ColumnSchema(
-            self.name,
-            tags=self.tags,
-            properties=new_properties,
-            dtype=self.dtype,
-            is_list=self.is_list,
-            is_ragged=is_ragged,
-        )
+        if value_counts:
+            return replace(
+                self,
+                properties=new_properties,
+                dtype=self.dtype.without_shape,
+                is_list=None,
+                is_ragged=None,
+            )
+        else:
+            return replace(
+                self,
+                properties=new_properties,
+            )
 
     def with_dtype(self, dtype, is_list: bool = None, is_ragged: bool = None) -> "ColumnSchema":
         """Create a copy of this ColumnSchema object with different column dtype
@@ -245,29 +215,54 @@ class ColumnSchema:
             Copied object with new column dtype
 
         """
-        is_list = is_list if is_list is not None else self.is_list
+        new_dtype = md.dtype(dtype).with_shape(self.shape)
 
-        if is_list:
-            is_ragged = is_ragged if is_ragged is not None else self.is_ragged
-        else:
-            is_ragged = False
+        properties = self.properties.copy()
+        if is_list is not None or is_ragged is not None:
+            properties.pop("value_count", None)
+            new_dtype = new_dtype.without_shape
 
-        return ColumnSchema(
-            self.name,
-            tags=self.tags,
-            properties=self.properties,
-            dtype=dtype,
-            is_list=is_list,
-            is_ragged=is_ragged,
+        return replace(
+            self, dtype=new_dtype, properties=properties, is_list=is_list, is_ragged=is_ragged
+        )
+
+    def with_shape(self, shape: Union[Tuple, Shape]) -> "ColumnSchema":
+        """
+        Create a copy of this object with a new shape
+
+        Parameters
+        ----------
+        shape : Union[Tuple, Shape]
+            Object to set as shape, must be either a tuple or Shape.
+
+        Returns
+        -------
+        ColumnSchema
+            A copy of this object containing the provided shape value
+
+        Raises
+        ------
+        TypeError
+            If value is not either a tuple or a Shape
+        """
+        dims = Shape(shape).as_tuple
+        properties = self.properties.copy()
+        properties.pop("value_count", None)
+        return replace(
+            self,
+            dims=dims,
+            properties=properties,
+            is_list=None,
+            is_ragged=None,
         )
 
     @property
     def int_domain(self) -> Optional[Domain]:
-        return self._domain() if np.issubdtype(self.dtype, np.integer) else None
+        return self._domain() if self.dtype.is_integer else None
 
     @property
     def float_domain(self) -> Optional[Domain]:
-        return self._domain() if np.issubdtype(self.dtype, np.floating) else None
+        return self._domain() if self.dtype.is_float else None
 
     @property
     def value_count(self) -> Optional[Domain]:
@@ -275,12 +270,13 @@ class ColumnSchema:
         return Domain(**value_count) if value_count else None
 
     def __merge__(self, other):
-        col_schema = self.with_tags(other.tags)
-        col_schema = col_schema.with_properties(other.properties)
-        col_schema = col_schema.with_dtype(
-            other.dtype, is_list=other.is_list, is_ragged=other.is_ragged
+        col_schema = (
+            self.with_name(other.name)
+            .with_dtype(other.dtype)
+            .with_tags(other.tags)
+            .with_properties(other.properties)
+            .with_shape(other.shape)
         )
-        col_schema = col_schema.with_name(other.name)
         return col_schema
 
     def __str__(self) -> str:
@@ -291,6 +287,59 @@ class ColumnSchema:
         domain = self.properties.get("domain")
         return Domain(**domain) if domain else None
 
+    def _validate_shape_info(self, shape, value_counts, is_list, is_ragged):
+        value_counts = value_counts or {}
+
+        min_count = value_counts.get("min", None)
+        max_count = value_counts.get("max", None)
+        ragged_counts = min_count != max_count
+
+        if shape and shape.dims is not None:
+            if is_ragged is not None and shape.is_ragged != is_ragged:
+                raise ValueError(
+                    f"Provided value of `is_ragged={is_ragged}` "
+                    f"is inconsistent with shape `{shape}`."
+                )
+            elif is_list is not None and shape.is_list != is_list:
+                raise ValueError(
+                    f"Provided value of `is_list={is_list}` "
+                    f"is inconsistent with shape `{shape}`."
+                )
+
+        if value_counts and shape and shape.dims is not None:
+            if (min_count and min_count != shape.dims[1].min) or (
+                max_count and max_count != shape.dims[1].max
+            ):
+                raise ValueError(
+                    f"Provided value counts `{value_counts}` "
+                    f"are inconsistent with shape `{shape}`."
+                )
+
+        if is_list is False and is_ragged is True:
+            raise ValueError(
+                "Columns with `is_list=False` can't set `is_ragged=True`, "
+                "since non-list columns can't be ragged."
+            )
+
+        if value_counts and is_ragged is not None and is_ragged != ragged_counts:
+            raise ValueError(
+                f"Provided value of `is_ragged={is_ragged}` "
+                f"is inconsistent with value counts `{value_counts}`."
+            )
+
+        # TODO: Enable this validation once we've removed these cases
+        #       from downstream Merlin libraries
+        # if (
+        #     not value_counts
+        #     and not (shape and shape.dims)
+        #     and is_list is True
+        #     and is_ragged is False
+        # ):
+        #     raise ValueError(
+        #         "Can't determine a shape for this column from "
+        #         "`is_list=True` and `is_ragged=False` without value counts. "
+        #     )
+
 
 class Schema:
     """A collection of column schemas for a dataset."""
